==> edge-login-ui-rn-2.20.0.tgz: package/lib/components/scenes/PinLoginScene.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeNetworkError, asMaybePasswordError, asMaybeUsernameError } from 'edge-core-js';
import * as React from 'react';
import { FlatList, Keyboard, Platform, Text, TouchableOpacity, TouchableWithoutFeedback, View } from 'react-native';
import LinearGradient from 'react-native-linear-gradient';
import { cacheStyles } from 'react-native-patina';
import { SafeAreaView } from 'react-native-safe-area-context';
import { SvgXml } from 'react-native-svg';
import MaterialCommunityIcons from 'react-native-vector-icons/MaterialCommunityIcons';
import { sprintf } from 'sprintf-js';
import { completeLogin } from '../../actions/LoginCompleteActions';
import { FaceIdXml } from '../../assets/xml/FaceId';
import { lstrings } from '../../common/locales/strings';
import { useImports } from '../../hooks/useImports';
import { useLocalUsers } from '../../hooks/useLocalUsers';
import { getLoginKey } from '../../keychain';
import { useDispatch, useSelector } from '../../types/ReduxTypes';
import { FourDigitDisplay } from '../abSpecific/FourDigitDisplay';
import { LogoImageHeader } from '../abSpecific/LogoImageHeader';
import { PinKeypad } from '../abSpecific/PinKeypad';
import { UserListItem } from '../abSpecific/UserListItem';
import { EdgeAnim } from '../common/EdgeAnim';
import { ButtonsModal } from '../modals/ButtonsModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { ThemedScene } from '../themed/ThemedScene';
export function PinLoginScene(props) {
    const { branding, route } = props;
    const { loginId } = route.params;
    const { accountOptions, context, onLogEvent = (event, values) => { } } = useImports();
    const dispatch = useDispatch();
    const theme = useTheme();
    const styles = getStyles(theme);
    // ---------------------------------------------------------------------
    // State
    // ---------------------------------------------------------------------
    const [pin, setPin] = React.useState('');
    const [showUserList, setShowUserList] = React.useState(false);
    const [touchBusy, setTouchBusy] = React.useState(false);
    // Error state:
    const [errorInfo, setErrorInfo] = React.useState();
    const hasWait = errorInfo != null && errorInfo.wait > 0;
    // Pin login state:
    const touch = useSelector(state => state.touch.type);
    const isTouchIdDisabled = hasWait || pin.length === 4 || touchBusy;
    // User list:
    const localUsers = useLocalUsers();
    const dropdownItems = React.useMemo(() => localUsers.filter(user => user.pinLoginEnabled || (touch && user.touchLoginEnabled)), [touch, localUsers]);
    // Active user:
    const userInfo = React.useMemo(() => { var _a; return (_a = dropdownItems.find(user => user.loginId === loginId)) !== null && _a !== void 0 ? _a : dropdownItems[0]; }, [dropdownItems, loginId]);
    // ---------------------------------------------------------------------
    // Effects
    // ---------------------------------------------------------------------
    // Runs once at start:
    React.useEffect(() => {
        if (userInfo != null && touch !== 'FaceID') {
            handleTouchLogin(userInfo).catch(showError);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    React.useEffect(() => {
        var _a;
        if (userInfo == null ||
            (!userInfo.touchLoginEnabled && !userInfo.pinLoginEnabled)) {
            dispatch({
                type: 'NAVIGATE',
                data: {
                    name: 'passwordLogin',
                    params: { username: (_a = userInfo === null || userInfo === void 0 ? void 0 : userInfo.username) !== null && _a !== void 0 ? _a : '' }
                }
            });
        }
    }, [dispatch, userInfo]);
    // Countdown timer:
    React.useEffect(() => {
        if (errorInfo == null || errorInfo.wait <= 0)
            return;
        const id = setTimeout(() => {
            setErrorInfo(Object.assign(Object.assign({}, errorInfo), { wait: errorInfo.wait - 1 }));
        }, 1000);
        return () => clearTimeout(id);
    }, [errorInfo]);
    // ---------------------------------------------------------------------
    // Handlers
    // ---------------------------------------------------------------------
    const handleBack = () => {
        var _a;
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: 'passwordLogin',
                params: { username: (_a = userInfo === null || userInfo === void 0 ? void 0 : userInfo.username) !== null && _a !== void 0 ? _a : '' }
            }
        });
    };
    const handleDelete = (userInfo) => {
        setShowUserList(false);
        Keyboard.dismiss();
        Airship.show(bridge => {
            var _a;
            return (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.forget_account, message: sprintf(lstrings.forget_username_account, (_a = userInfo.username) !== null && _a !== void 0 ? _a : lstrings.username), buttons: {
                    ok: { label: lstrings.forget },
                    cancel: { label: lstrings.cancel, type: 'secondary' }
                } }));
        })
            .then((button) => __awaiter(this, void 0, void 0, function* () {
            if (button !== 'ok')
                return;
            if (context.forgetAccount != null) {
                yield context.forgetAccount(userInfo.loginId);
            }
            else {
                const { username } = userInfo;
                if (username == null)
                    throw new Error('No username');
                yield context.deleteLocalAccount(username);
            }
        }))
            .catch(showError);
    };
    const handlePinLogin = (userInfo, pin) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        try {
            const { loginId } = userInfo;
            const account = yield context.loginWithPIN(loginId, pin, Object.assign(Object.assign({}, accountOptions), { useLoginId: true }));
            yield dispatch(completeLogin(account));
        }
        catch (error) {
            console.log('LOG IN WITH PIN ERROR ', error);
            const passwordError = asMaybePasswordError(error);
            const usernameError = asMaybeUsernameError(error);
            const networkError = asMaybeNetworkError(error);
            setErrorInfo({
                message: passwordError != null
                    ? lstrings.invalid_pin
                    : usernameError != null
                        ? lstrings.pin_not_enabled
                        : networkError != null
                            ? translateNetworkError(networkError, userInfo)
                            : error instanceof Error
                                ? error.message
                                : String(error),
                wait: Math.ceil((_a = passwordError === null || passwordError === void 0 ? void 0 : passwordError.wait) !== null && _a !== void 0 ? _a : 0)
            });
            setPin('');
            setTouchBusy(false);
        }
    });
    const handleTouchLogin = (userInfo) => __awaiter(this, void 0, void 0, function* () {
        try {
            const { loginId, username = lstrings.missing_username } = userInfo;
            const loginKey = yield getLoginKey(userInfo, `Touch to login user: "${username}"`, lstrings.login_with_password);
            if (loginKey == null)
                return;
            setTouchBusy(true);
            const account = yield context.loginWithKey(loginId, loginKey, Object.assign(Object.assign({}, accountOptions), { useLoginId: true }));
            yield dispatch(completeLogin(account));
        }
        finally {
            setTouchBusy(false);
        }
    });
    const handleTouchId = () => {
        if (userInfo == null)
            return;
        handleTouchLogin(userInfo).catch(showError);
    };
    const handlePress = (value) => {
        const newPin = value === 'back' ? pin.slice(0, -1) : pin.concat(value).slice(0, 4);
        setPin(newPin);
        setErrorInfo(undefined);
        if (newPin.length === 4 && pin.length === 3 && userInfo != null) {
            handlePinLogin(userInfo, newPin)
                .then(() => {
                onLogEvent('Pin_Login');
            })
                .catch(showError);
        }
    };
    const handleSelectUser = (userInfo) => {
        setShowUserList(false);
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'pinLogin', params: { loginId: userInfo.loginId } }
        });
        setErrorInfo(undefined);
        handleTouchLogin(userInfo).catch(showError);
    };
    const handleShowDrop = () => {
        setShowUserList(true);
    };
    const handleHideDrop = () => {
        setShowUserList(false);
    };
    // ---------------------------------------------------------------------
    // Rendering
    // ---------------------------------------------------------------------
    const renderBottomHalf = () => {
        if (showUserList) {
            return (React.createElement(View, { style: styles.innerView },
                React.createElement(FlatList, { style: styles.listView, data: dropdownItems, renderItem: renderItems, keyExtractor: item => item.loginId })));
        }
        let errorMessage = '';
        if (errorInfo != null) {
            errorMessage = errorInfo.message;
            if (errorInfo.wait > 0) {
                errorMessage +=
                    ': ' + sprintf(lstrings.account_locked_for, errorInfo.wait);
            }
        }
        const isSingleSavedUser = dropdownItems.length === 1;
        let usernameLabel = '';
        if ((userInfo === null || userInfo === void 0 ? void 0 : userInfo.username) != null) {
            // Normal account: show username
            usernameLabel = userInfo.username;
        }
        else if (!isSingleSavedUser) {
            // Light account + other saved users: "Tap to Switch..."
            usernameLabel = lstrings.tap_to_switch_user;
        }
        // Light account + no other saved users: hide username label
        return (React.createElement(View, { style: styles.innerView },
            React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 40 } },
                React.createElement(TouchableOpacity, { testID: "usernameDropdownButton", style: styles.usernameShadow, onPress: isSingleSavedUser ? undefined : handleShowDrop, activeOpacity: isSingleSavedUser ? 1 : undefined },
                    React.createElement(LinearGradient, { colors: theme.pinUsernameButton, start: theme.pinUsernameButtonColorStart, end: theme.pinUsernameButtonColorEnd, style: [styles.linearGradient, styles.usernameButton] },
                        React.createElement(Text, { adjustsFontSizeToFit: true, minimumFontScale: 0.75, numberOfLines: 1, style: styles.usernameText }, usernameLabel)))),
            userInfo == null || !userInfo.pinLoginEnabled ? (React.createElement(View, { style: styles.spacer })) : (React.createElement(FourDigitDisplay, { error: errorMessage, pin: pin, spinner: hasWait || pin.length === 4 || touchBusy })),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 20 } }, renderTouchImage()),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 40 } },
                React.createElement(Text, { style: styles.touchImageText }, renderTouchImageText()))));
    };
    const renderItems = (item) => {
        return (React.createElement(UserListItem, { userInfo: item.item, onClick: handleSelectUser, onDelete: handleDelete, 
            // TODO: New dropdown visual design for this scene OR
            // New design for combined password/pin scene
            onLayout: () => { } }));
    };
    const renderTouchImage = () => {
        if (userInfo == null || !userInfo.touchLoginEnabled)
            return null;
        if (touch === 'FaceID') {
            return (React.createElement(TouchableOpacity, { onPress: handleTouchId, disabled: isTouchIdDisabled },
                React.createElement(SvgXml, { xml: FaceIdXml, color: theme.iconTappable, width: theme.rem(3), height: theme.rem(3) })));
        }
        if (touch === 'TouchID') {
            return (React.createElement(TouchableOpacity, { onPress: handleTouchId, disabled: isTouchIdDisabled },
                React.createElement(MaterialCommunityIcons, { name: "fingerprint", size: theme.rem(3), color: theme.iconTappable })));
        }
        return null;
    };
    const renderTouchImageText = () => {
        if (userInfo == null || !userInfo.touchLoginEnabled)
            return '';
        if (touch === 'FaceID') {
            return lstrings.use_faceId;
        }
        if (touch === 'TouchID' && Platform.OS === 'ios') {
            return lstrings.use_touchId;
        }
        if (touch === 'TouchID' && Platform.OS !== 'ios') {
            return lstrings.use_fingerprint;
        }
        return '';
    };
    return (React.createElement(ThemedScene, { backButtonText: lstrings.exit_pin, onBack: handleBack, noUnderline: true, branding: branding },
        React.createElement(View, { style: styles.featureBoxContainer },
            React.createElement(TouchableWithoutFeedback, { accessible: false, onPress: handleHideDrop },
                React.createElement(View, { style: styles.featureBox },
                    React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 60 } },
                        React.createElement(LogoImageHeader, { branding: branding })),
                    React.createElement(View, { style: styles.featureBoxBody }, renderBottomHalf()))),
            React.createElement(View, { style: styles.spacer_full }),
            userInfo == null || !userInfo.pinLoginEnabled ? null : (React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 40 } },
                React.createElement(PinKeypad, { disabled: hasWait || pin.length === 4, onPress: handlePress }))),
            React.createElement(SafeAreaView, { edges: ['bottom'] }))));
}
function translateNetworkError(error, userInfo) {
    if (userInfo.username != null) {
        return `${error.message} ${lstrings.pin_network_error_full_password}`;
    }
    if (userInfo.keyLoginEnabled) {
        return `${error.message} ${lstrings.pin_network_error_biometric}`;
    }
    return error.message;
}
const getStyles = cacheStyles((theme) => ({
    listView: {
        height: theme.rem(16),
        width: theme.rem(10),
        backgroundColor: theme.modal
    },
    featureBoxContainer: {
        width: '100%',
        flex: 1,
        flexDirection: 'column',
        justifyContent: 'space-around'
    },
    featureBox: {
        width: '100%',
        alignItems: 'center'
    },
    featureBoxBody: {
        height: theme.rem(15),
        width: '100%'
    },
    innerView: {
        height: '100%',
        width: '100%',
        justifyContent: 'flex-start',
        alignItems: 'center'
    },
    touchImageText: {
        marginTop: theme.rem(0.5),
        color: theme.iconTappable
    },
    usernameText: {
        fontFamily: theme.pinUsernameButtonFont,
        color: theme.pinUsernameButtonText,
        fontSize: theme.rem(theme.pinUsernameButtonFontSizeRem),
        paddingHorizontal: theme.rem(1),
        paddingVertical: theme.rem(0.5)
    },
    usernameButton: {
        borderColor: theme.pinUsernameButtonOutline,
        borderWidth: theme.pinUsernameButtonOutlineWidth
    },
    usernameShadow: Object.assign({}, theme.pinUsernameButtonShadow),
    linearGradient: {
        // flex: 1,
        paddingLeft: 0,
        paddingRight: 0,
        borderRadius: theme.rem(theme.pinUsernameButtonBorderRadiusRem)
    },
    spacer: {
        marginTop: theme.rem(2)
    },
    spacer_full: {
        flex: 1,
        zIndex: -100
    }
}));

==> edge-login-ui-rn-3.3.1.tgz: package/lib/components/scenes/PinLoginScene.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeNetworkError, asMaybePasswordError, asMaybeUsernameError } from 'edge-core-js';
import * as React from 'react';
import { FlatList, Keyboard, Platform, Text, TouchableOpacity, TouchableWithoutFeedback, View } from 'react-native';
import LinearGradient from 'react-native-linear-gradient';
import { cacheStyles } from 'react-native-patina';
import { SafeAreaView } from 'react-native-safe-area-context';
import { SvgXml } from 'react-native-svg';
import MaterialCommunityIcons from 'react-native-vector-icons/MaterialCommunityIcons';
import { sprintf } from 'sprintf-js';
import { completeLogin } from '../../actions/LoginCompleteActions';
import { FaceIdXml } from '../../assets/xml/FaceId';
import { lstrings } from '../../common/locales/strings';
import { useImports } from '../../hooks/useImports';
import { useLocalUsers } from '../../hooks/useLocalUsers';
import { getLoginKey } from '../../keychain';
import { useDispatch, useSelector } from '../../types/ReduxTypes';
import { FourDigitDisplay } from '../abSpecific/FourDigitDisplay';
import { LogoImageHeader } from '../abSpecific/LogoImageHeader';
import { PinKeypad } from '../abSpecific/PinKeypad';
import { UserListItem } from '../abSpecific/UserListItem';
import { EdgeAnim } from '../common/EdgeAnim';
import { ButtonsModal } from '../modals/ButtonsModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { ThemedScene } from '../themed/ThemedScene';
export function PinLoginScene(props) {
    const { branding, route } = props;
    const { loginId } = route.params;
    const { accountOptions, context, onLogEvent = () => { } } = useImports();
    const dispatch = useDispatch();
    const theme = useTheme();
    const styles = getStyles(theme);
    // ---------------------------------------------------------------------
    // State
    // ---------------------------------------------------------------------
    const [pin, setPin] = React.useState('');
    const [showUserList, setShowUserList] = React.useState(false);
    const [touchBusy, setTouchBusy] = React.useState(false);
    // Error state:
    const [errorInfo, setErrorInfo] = React.useState();
    const hasWait = errorInfo != null && errorInfo.wait > 0;
    // Pin login state:
    const touch = useSelector(state => state.touch.type);
    const isTouchIdDisabled = hasWait || pin.length === 4 || touchBusy;
    // User list:
    const localUsers = useLocalUsers();
    const dropdownItems = React.useMemo(() => localUsers.filter(user => user.pinLoginEnabled || (touch && user.touchLoginEnabled)), [touch, localUsers]);
    // Active user:
    const userInfo = React.useMemo(() => { var _a; return (_a = dropdownItems.find(user => user.loginId === loginId)) !== null && _a !== void 0 ? _a : dropdownItems[0]; }, [dropdownItems, loginId]);
    // ---------------------------------------------------------------------
    // Effects
    // ---------------------------------------------------------------------
    // Runs once at start:
    React.useEffect(() => {
        if (userInfo != null && touch !== 'FaceID') {
            handleTouchLogin(userInfo).catch(showError);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    React.useEffect(() => {
        var _a;
        if (userInfo == null ||
            (!userInfo.touchLoginEnabled && !userInfo.pinLoginEnabled)) {
            dispatch({
                type: 'NAVIGATE',
                data: {
                    name: 'passwordLogin',
                    params: { username: (_a = userInfo === null || userInfo === void 0 ? void 0 : userInfo.username) !== null && _a !== void 0 ? _a : '' }
                }
            });
        }
    }, [dispatch, userInfo]);
    // Countdown timer:
    React.useEffect(() => {
        if (errorInfo == null || errorInfo.wait <= 0)
            return;
        const id = setTimeout(() => {
            setErrorInfo(Object.assign(Object.assign({}, errorInfo), { wait: errorInfo.wait - 1 }));
        }, 1000);
        return () => clearTimeout(id);
    }, [errorInfo]);
    // ---------------------------------------------------------------------
    // Handlers
    // ---------------------------------------------------------------------
    const handleBack = () => {
        var _a;
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: 'passwordLogin',
                params: { username: (_a = userInfo === null || userInfo === void 0 ? void 0 : userInfo.username) !== null && _a !== void 0 ? _a : '' }
            }
        });
    };
    const handleDelete = (userInfo) => {
        setShowUserList(false);
        Keyboard.dismiss();
        Airship.show(bridge => {
            var _a;
            return (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.forget_account, message: sprintf(lstrings.forget_username_account, (_a = userInfo.username) !== null && _a !== void 0 ? _a : lstrings.username), buttons: {
                    ok: { label: lstrings.forget },
                    cancel: { label: lstrings.cancel, type: 'secondary' }
                } }));
        })
            .then((button) => __awaiter(this, void 0, void 0, function* () {
            if (button !== 'ok')
                return;
            if (context.forgetAccount != null) {
                yield context.forgetAccount(userInfo.loginId);
            }
            else {
                const { username } = userInfo;
                if (username == null)
                    throw new Error('No username');
                yield context.deleteLocalAccount(username);
            }
        }))
            .catch(showError);
    };
    const handlePinLogin = (userInfo, pin) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        try {
            const { loginId } = userInfo;
            const account = yield context.loginWithPIN(loginId, pin, Object.assign(Object.assign({}, accountOptions), { useLoginId: true }));
            yield dispatch(completeLogin(account));
        }
        catch (error) {
            console.log('LOG IN WITH PIN ERROR ', error);
            const passwordError = asMaybePasswordError(error);
            const usernameError = asMaybeUsernameError(error);
            const networkError = asMaybeNetworkError(error);
            setErrorInfo({
                message: passwordError != null
                    ? lstrings.invalid_pin
                    : usernameError != null
                        ? lstrings.pin_not_enabled
                        : networkError != null
                            ? translateNetworkError(networkError, userInfo)
                            : error instanceof Error
                                ? error.message
                                : String(error),
                wait: Math.ceil((_a = passwordError === null || passwordError === void 0 ? void 0 : passwordError.wait) !== null && _a !== void 0 ? _a : 0)
            });
            setPin('');
            setTouchBusy(false);
        }
    });
    const handleTouchLogin = (userInfo) => __awaiter(this, void 0, void 0, function* () {
        try {
            const { loginId, username = lstrings.missing_username } = userInfo;
            const loginKey = yield getLoginKey(userInfo, `Touch to login user: "${username}"`, lstrings.login_with_password);
            if (loginKey == null)
                return;
            setTouchBusy(true);
            const account = yield context.loginWithKey(loginId, loginKey, Object.assign(Object.assign({}, accountOptions), { useLoginId: true }));
            yield dispatch(completeLogin(account));
        }
        finally {
            setTouchBusy(false);
        }
    });
    const handleTouchId = () => {
        if (userInfo == null)
            return;
        handleTouchLogin(userInfo).catch(showError);
    };
    const handlePress = (value) => {
        const newPin = value === 'back' ? pin.slice(0, -1) : pin.concat(value).slice(0, 4);
        setPin(newPin);
        setErrorInfo(undefined);
        if (newPin.length === 4 && pin.length === 3 && userInfo != null) {
            handlePinLogin(userInfo, newPin)
                .then(() => {
                onLogEvent('Pin_Login');
            })
                .catch(showError);
        }
    };
    const handleSelectUser = (userInfo) => {
        setShowUserList(false);
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'pinLogin', params: { loginId: userInfo.loginId } }
        });
        setErrorInfo(undefined);
        handleTouchLogin(userInfo).catch(showError);
    };
    const handleShowDrop = () => {
        setShowUserList(true);
    };
    const handleHideDrop = () => {
        setShowUserList(false);
    };
    // ---------------------------------------------------------------------
    // Rendering
    // ---------------------------------------------------------------------
    const renderBottomHalf = () => {
        if (showUserList) {
            return (React.createElement(View, { style: styles.innerView },
                React.createElement(FlatList, { style: styles.listView, data: dropdownItems, renderItem: renderItems, keyExtractor: item => item.loginId })));
        }
        let errorMessage = '';
        if (errorInfo != null) {
            errorMessage = errorInfo.message;
            if (errorInfo.wait > 0) {
                errorMessage +=
                    ': ' + sprintf(lstrings.account_locked_for, errorInfo.wait);
            }
        }
        const isSingleSavedUser = dropdownItems.length === 1;
        let usernameLabel = '';
        if ((userInfo === null || userInfo === void 0 ? void 0 : userInfo.username) != null) {
            // Normal account: show username
            usernameLabel = userInfo.username;
        }
        else if (!isSingleSavedUser) {
            // Light account + other saved users: "Tap to Switch..."
            usernameLabel = lstrings.tap_to_switch_user;
        }
        // Light account + no other saved users: hide username label
        return (React.createElement(View, { style: styles.innerView },
            React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 40 } },
                React.createElement(TouchableOpacity, { testID: "usernameDropdownButton", style: styles.usernameShadow, onPress: isSingleSavedUser ? undefined : handleShowDrop, activeOpacity: isSingleSavedUser ? 1 : undefined },
                    React.createElement(LinearGradient, { colors: theme.pinUsernameButton, start: theme.pinUsernameButtonColorStart, end: theme.pinUsernameButtonColorEnd, style: [styles.linearGradient, styles.usernameButton] },
                        React.createElement(Text, { adjustsFontSizeToFit: true, minimumFontScale: 0.75, numberOfLines: 1, style: styles.usernameText }, usernameLabel)))),
            userInfo == null || !userInfo.pinLoginEnabled ? (React.createElement(View, { style: styles.spacer })) : (React.createElement(FourDigitDisplay, { error: errorMessage, pin: pin, spinner: hasWait || pin.length === 4 || touchBusy })),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 20 } }, renderTouchImage()),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 40 } },
                React.createElement(Text, { style: styles.touchImageText }, renderTouchImageText()))));
    };
    const renderItems = (item) => {
        return (React.createElement(UserListItem, { userInfo: item.item, onClick: handleSelectUser, onDelete: handleDelete, 
            // TODO: New dropdown visual design for this scene OR
            // New design for combined password/pin scene
            onLayout: () => { } }));
    };
    const renderTouchImage = () => {
        if (userInfo == null || !userInfo.touchLoginEnabled)
            return null;
        if (touch === 'FaceID') {
            return (React.createElement(TouchableOpacity, { onPress: handleTouchId, disabled: isTouchIdDisabled },
                React.createElement(SvgXml, { xml: FaceIdXml, color: theme.iconTappable, width: theme.rem(3), height: theme.rem(3) })));
        }
        if (touch === 'TouchID') {
            return (React.createElement(TouchableOpacity, { onPress: handleTouchId, disabled: isTouchIdDisabled },
                React.createElement(MaterialCommunityIcons, { name: "fingerprint", size: theme.rem(3), color: theme.iconTappable })));
        }
        return null;
    };
    const renderTouchImageText = () => {
        if (userInfo == null || !userInfo.touchLoginEnabled)
            return '';
        if (touch === 'FaceID') {
            return lstrings.use_faceId;
        }
        if (touch === 'TouchID' && Platform.OS === 'ios') {
            return lstrings.use_touchId;
        }
        if (touch === 'TouchID' && Platform.OS !== 'ios') {
            return lstrings.use_fingerprint;
        }
        return '';
    };
    return (React.createElement(ThemedScene, { backButtonText: lstrings.exit_pin, onBack: handleBack, noUnderline: true, branding: branding },
        React.createElement(View, { style: styles.featureBoxContainer },
            React.createElement(TouchableWithoutFeedback, { accessible: false, onPress: handleHideDrop },
                React.createElement(View, { style: styles.featureBox },
                    React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 60 } },
                        React.createElement(LogoImageHeader, { branding: branding })),
                    React.createElement(View, { style: styles.featureBoxBody }, renderBottomHalf()))),
            React.createElement(View, { style: styles.spacer_full }),
            userInfo == null || !userInfo.pinLoginEnabled ? null : (React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 40 } },
                React.createElement(PinKeypad, { disabled: hasWait || pin.length === 4, onPress: handlePress }))),
            React.createElement(SafeAreaView, { edges: ['bottom'] }))));
}
function translateNetworkError(error, userInfo) {
    if (userInfo.username != null) {
        return `${error.message} ${lstrings.pin_network_error_full_password}`;
    }
    if (userInfo.keyLoginEnabled) {
        return `${error.message} ${lstrings.pin_network_error_biometric}`;
    }
    return error.message;
}
const getStyles = cacheStyles((theme) => ({
    listView: {
        height: theme.rem(16),
        width: theme.rem(10),
        backgroundColor: theme.modal
    },
    featureBoxContainer: {
        width: '100%',
        flex: 1,
        flexDirection: 'column',
        justifyContent: 'space-around'
    },
    featureBox: {
        width: '100%',
        alignItems: 'center'
    },
    featureBoxBody: {
        height: theme.rem(15),
        width: '100%'
    },
    innerView: {
        height: '100%',
        width: '100%',
        justifyContent: 'flex-start',
        alignItems: 'center'
    },
    touchImageText: {
        marginTop: theme.rem(0.5),
        color: theme.iconTappable
    },
    usernameText: {
        fontFamily: theme.pinUsernameButtonFont,
        color: theme.pinUsernameButtonText,
        fontSize: theme.rem(theme.pinUsernameButtonFontSizeRem),
        paddingHorizontal: theme.rem(1),
        paddingVertical: theme.rem(0.5)
    },
    usernameButton: {
        borderColor: theme.pinUsernameButtonOutline,
        borderWidth: theme.pinUsernameButtonOutlineWidth
    },
    usernameShadow: Object.assign({}, theme.pinUsernameButtonShadow),
    linearGradient: {
        // flex: 1,
        paddingLeft: 0,
        paddingRight: 0,
        borderRadius: theme.rem(theme.pinUsernameButtonBorderRadiusRem)
    },
    spacer: {
        marginTop: theme.rem(2)
    },
    spacer_full: {
        flex: 1,
        zIndex: -100
    }
}));
